﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Diagnostics;

namespace ComparisonSpeed {
	class Program {
		const int Items = 1000000;

		static Random r = new Random();

		static List<string> GenerateStrings() {
			Console.WriteLine("Generating strings ...");
			byte[] buffer = new byte[10];
			var ls = new List<string>(Items);
			for (int i = 0; i < Items; i++) {
				r.NextBytes(buffer);
				ls.Add(Encoding.ASCII.GetString(buffer));
			}
			return ls;
		}

		static void Main(string[] args) {
			Console.WriteLine("Generating ints ...");
			var li = new List<int>(Items);
			for (int i = 0; i < Items; i++) {
				li.Add(r.Next(int.MaxValue));
			}

			Console.WriteLine("Sorting ints ...");
			Stopwatch sw = Stopwatch.StartNew();
			li.Sort();
			sw.Stop();

			Console.WriteLine("Sort of ints: {0}", sw.Elapsed.TotalSeconds);

			//
			//

			System.Threading.Thread.CurrentThread.CurrentCulture = new System.Globalization.CultureInfo("en-US");
			var ls = GenerateStrings();
			Console.WriteLine("Sorting strings ...");
			sw = Stopwatch.StartNew();
			ls.Sort();
			sw.Stop();

			Console.WriteLine("Sort (en-US) of strings: {0}", sw.Elapsed.TotalSeconds);
			
			//
			//

			System.Threading.Thread.CurrentThread.CurrentCulture = new System.Globalization.CultureInfo("cs-CZ");
			ls = GenerateStrings();
			Console.WriteLine("Sorting strings ...");
			sw = Stopwatch.StartNew();
			ls.Sort();
			sw.Stop();

			Console.WriteLine("Sort (cs-CZ) of strings: {0}", sw.Elapsed.TotalSeconds);
			Console.ReadLine();

			//
			//

			ls = GenerateStrings();
			Console.WriteLine("Sorting strings ...");
			sw = Stopwatch.StartNew();
			ls.Sort(StringComparer.Ordinal);
			sw.Stop();

			Console.WriteLine("Ordinal sort of strings: {0}", sw.Elapsed.TotalSeconds);
			Console.ReadLine();

			//
			//

			ls = GenerateStrings();
			Console.WriteLine("Sorting strings ...");
			sw = Stopwatch.StartNew();
			ls.Sort((s1, s2) => s1.ToUpper().CompareTo(s2.ToUpper()));
			sw.Stop();

			Console.WriteLine("Case insensitive via ToUpper() sort of strings: {0}", sw.Elapsed.TotalSeconds);
			Console.ReadLine();

			//
			//

			ls = GenerateStrings();
			Console.WriteLine("Sorting strings ...");
			sw = Stopwatch.StartNew();
			ls.Sort(StringComparer.CurrentCultureIgnoreCase);
			sw.Stop();

			Console.WriteLine("CurrentCultureIgnoreCase sort of strings: {0}", sw.Elapsed.TotalSeconds);
			
			//
			//

			Random r2 = new Random();
			byte[] buffer = new byte[10];

			Console.WriteLine("Generating Ps ...");
			var lp = new List<P>(Items);
			for (int i = 0; i < Items; i++) {
				var p = new P();
				r.NextBytes(buffer);
				p.FirstName = Encoding.ASCII.GetString(buffer);
				r.NextBytes(buffer);
				p.LastName = Encoding.ASCII.GetString(buffer);
				p.Level = r.Next(2);
				lp.Add(p);
			}

			Console.WriteLine("Sorting Ps ...");
			sw = Stopwatch.StartNew();
			lp.Sort();
			sw.Stop();

			Console.WriteLine("Sort of Ps: {0}", sw.Elapsed.TotalSeconds);
		}

		class P : IComparable<P> {
			public string FirstName { get; set; }
			public string LastName { get; set; }
			public int Level { get; set; }

			public int CompareTo(P p2) {
				var levelResult = Level.CompareTo(p2.Level);
				if (levelResult != 0) {
					return levelResult;
				} else {
					var lastNameResult = LastName.CompareTo(p2.LastName);
					if (lastNameResult != 0) {
						return lastNameResult;
					} else {
						return FirstName.CompareTo(p2.FirstName);
					}
				}
			}
		}
	}


}
